# Copyright 2011-2015 梅濁酒(umedoblock)

import sys
import sqlite3
import configparser
import os, sys
from collections import namedtuple
from collections import OrderedDict

_here = os.path.abspath(__file__)
_parent_dir = os.path.join(os.path.dirname(_here), "..")
# print("_parent_dir =", _parent_dir)
sys.path.append(_parent_dir)

from iasap.lib import logger

class GeneralSQLConnection(object):
    def __init__(self, conn):
        self.conn = conn
        try:
            self.create_namedtuples_table()
        except sqlite3.OperationalError:
            pass
        logger.info('GeneralSQL(self=0x{:x}, conn=0x{:x}) done.'. \
                          format(id(self), id(conn)))

    def commit(self):
        self.conn.commit()

    def close(self):
        self.commit()
        self.conn.close()
        logger.info('close {}.'.format(self))

    def create_namedtuples_table(self):
        conn = self.conn
        table_name = '__namedtuples__'
        sql = '''
            create table __namedtuples__
            (
                id integer primary key,
                typename uniq text,
                field_names text
            )'''
        logger.debug(sql)
        conn.execute(sql)
        conn.commit()
        logger.info('created __namedtuples__ table.')

        od = OrderedDict()
        od['id'] = None
        od['typename'] = table_name
        od['field_names'] = ''
        od['field_names'] = ' '.join(od.keys())
        logger.debug('od.keys() = {}'.format(od.keys))
        self.insert('__namedtuples__', od)
        logger.info('1: inserted __namedtuples__ table info '
                    'to __namedtuples__ table.')

    def create_table(self, table_info):
        cur = self.conn.cursor()
        table_name = table_info["table_name"]
        columns = []
        for column_name, explain in table_info.items():
            if column_name == "table_name":
                continue
            column = ' '.join([column_name, explain])
            columns += [column]
        sql = 'create table {} ({})'.format(table_name, ', '.join(columns))
        logger.debug(sql)
        cur.execute(sql)
        logger.info('created {} table.'.format(table_name))

        # print(list(table_info.keys()))
        od = OrderedDict()
        od['id'] = None
        od['typename'] = table_name
        L = list(table_info.keys())
        logger.debug('list(table_info.keys()) = {}'.format(L))
        L.remove("table_name")
        od['field_names'] = ' '.join(L)
        self.insert('__namedtuples__', od)
        logger.info('2: inserted {} table info '
                    'to __namedtuples__ table.'.format(table_name))

    def insert(self, table_name, columns=None):
        cur = self.conn.cursor()
        hatenas = '({})'.format(', '.join('?' * len(columns.values())))
        sql_ = 'insert into {} {} values '.format(table_name, \
                                                  tuple(columns.keys()))
        values = tuple(columns.values())
        logger.debug(sql_ + str(values))
        cur.execute(sql_ + hatenas, values)
        cur.close()

    def select(self, sql, values=None, max_rows=0):
        logger.debug(sql)
        logger.debug(values)

        return NamedRow(self.conn, sql, values, max_rows)
#       named_rows = []
#       rows = cur.fetchall()
#       cur.close()
#       for row in rows:
#           named_row = namedtup(*row)
#           named_rows.append(named_row)

#       return named_rows

class NamedRow(object):
    def __init__(self, conn, sql, values=None, max_rows=0):
        self.sql = sql
        self.values = values
        self.cur = conn.cursor()
        self.max_rows = max_rows
        self._i = 0
        sp = sql.split()
        table_name_index = sp.index('from') + 1
        table_name = sp[table_name_index]

        namedtup = self._get_namedtuple(table_name)

        if self.values:
            result = self.cur.execute('select ' + sql, self.values)
        else:
            result = self.cur.execute('select ' + sql)
        self.namedtup = namedtup
        logger.info('NamedRow(self={})'.format(self))

    def _get_namedtuple(self, typename):
        if not hasattr(self.__dict__, typename):
            cur = self.cur
            sql = ('select * from __namedtuples__ where '
                   'typename = "{}"').format(typename)
            logger.debug(sql)
            result = cur.execute(sql)
            logger.info('got {} __namedtuples__ info.'.format(typename))
            row = cur.fetchone()
            if not row:
                raise ValueError("typename={} is not in __namedtuples__")
            field_names = row[2]
            namedtup = namedtuple(typename, field_names)
            self.__dict__[typename] = namedtup
        else:
            namedtup = self.__dict__[typename]
        return namedtup

    def __iter__(self):
        return self

    def __next__(self):
        self._i += 1
        row = None
        if not self.max_rows or self._i <= self.max_rows:
            row = self.cur.fetchone()
        if row:
            logger.debug("row = {}".format(row))
            logger.debug("self.namedtup = {}".format(self.namedtup))
            return self.namedtup(*row)
        else:
            self.cur.close()
            raise StopIteration

if __name__ == '__main__':
    import os
    abs_path = os.path.abspath(__file__)
    print(abs_path)
    dname = os.path.dirname(abs_path)
    print(dname)
    conf_path = os.path.join(dname, "iasap.conf")
    config = configparser.ConfigParser()
    config.read_file(open(conf_path), 'iasap')

    path = ':memory:'
    conn = sqlite3.connect(path)

    conn = GeneralSQLConnection(conn)
    conn.create_table(config['schema'])
    for id in range(0, 4):
        column = \
            {'id': None, 'key': 'key' + str(id), 'value': 'value' + str(id)}
        conn.insert('iasap', column)
        print('id =', id, column)
#   conn.commit()

    named_rows = conn.select('* from iasap')
    print('named_rows =', named_rows)
    for i, named_row in enumerate(named_rows):
        nr = named_row
        print('i =', i, nr)
      # print('i =', i, nr.id, nr.key, nr.value)
    conn.close()



# Copyright 2011-2015 梅濁酒(umedoblock)

import sys
import time
import unittest
import threading
import os
import tempfile

from iasap_curses import *

class TestIasapCurses(unittest.TestCase):
    def setUp(self):
        self.f = tempfile.TemporaryFile(dir='.')
        stdin_fileno = sys.stdin.fileno()
        os.dup2(self.f.fileno(), stdin_fileno)

    def write(self, ss):
        self.f.write(ss)
        self.f.seek(-len(ss), os.SEEK_CUR)

    def tearDown(self):
        stdin_fileno = sys.stdin.fileno()
        os.dup2(stdin_fileno, self.f.fileno())
        self.f.close()

    def test_start_and_end(self):
        ocs = IasapCurses()
        self.write(b'abc')
        ocs.run(CursesLine)

#   infinite loop
#   def test_over_write(self):
#       ocs = IasapCurses()
#       self.write(b'abc')
#       ocs.run(CursesLine)

if __name__ == '__main__':
    unittest.main()



import os, sys

def sys_path_append_parent_dir(file_path, n_up):
    if os.path.islink(file_path):
        file_path = os.path.realpath(file_path)
    here = os.path.abspath(file_path)
    ups = ""
    if n_up:
        ups = os.path.join(*([".."] * n_up))
    parent_dir = os.path.join(os.path.dirname(here), ups)
  # print("parent_dir =", parent_dir)
    sys.path.append(parent_dir)



# Copyright 2011-2015 梅濁酒(umedoblock)

import sqlite3, os, sys, datetime, re

from iasap.iasap_tkinter import IasapTkinter
from iasap.iasap_curses import IasapCurses
from iasap.sql import GeneralSQLConnection
from iasap.lib import logger, start_logger
from iasap.lib import merge_kv_by_defaults_and_argument, set_kv_for_regular

if os.path.islink(__file__):
    __file__ = os.path.realpath(__file__)
__file__ = os.path.abspath(__file__)
dirname = os.path.dirname(__file__)

class Iasap(object):
    DEFAULTS = {
        "conf": os.path.join(dirname, "..", "iasap.conf"),
        "dbpath": os.path.join(dirname, "..", 'iasap.sqlite3'),
        "mode": "tkinter",
        "limit": 30,
        "debug": False,
    }

    def __init__(self, dbpath, table_name, mode, height):
        limit = height
        self._limit = height

        try:
            # dbpath = ':memory:'
            conn = sqlite3.connect(dbpath)
        except sqlite3.OperationalError as raiz:
            if raiz.args[0] == 'unable to open database file':
                s = traceback.format_exc()
                logger.error(s)
                logger.error('dbpath is {}'.format(dbpath), file=sys.stderr)
            raise sqlite3.OperationalError(*raiz.args)

        conn = GeneralSQLConnection(conn)
      # print('conn =', conn)
        self.conn = conn

        self._make_sql_template(table_name, limit)

        logger.debug("mode is {} in Iasap.__init__()".format(mode))
        if mode == 'curses':
            _iasap = IasapCurses()
        elif mode == 'tkinter':
            _iasap = IasapTkinter()
        elif mode == 'one-shot':
            _iasap = self
        else:
            logger.debug("mode = \"{}\"".format(mode))
            raise ValueError('mode="{}" must be "curses" or "tkinter".'.format(mode))
        _iasap.get_body = self.get_body
        self.start = _iasap.start

    def get_body(self, search):
        sql, like_search = self._make_sql(search)
        tup = (like_search,)
        rows = self._do_select(sql, tup)
        logger.debug('like_search = "{}"'.format(like_search))
        if not search:
            return ''
        body = '\n'.join([text for text in self])
        return body

    def start(self, search):
        logger.debug("search: {}".format(search))
        body = self.get_body(search)
        for line in body.split("\n"):
            print(line)

    def _do_select(self, sql, values):
        logger.debug('sql = "{}"'.format(sql))
        logger.debug('values = "{}"'.format(values))
        s = datetime.datetime.now()
        self.named_rows = self.conn.select(sql, values)
        e = datetime.datetime.now()
        d = (e - s).total_seconds()
        logger.debug('select start at {}'.format(s))
        logger.debug('select   end at {}'.format(e))
        logger.debug('passed  time is {}'.format(d))

    def _make_like(self, search):
        """
        >>> import re
        >>> re.sub(" $", "%", " a ")
        ' a%'
        >>> re.sub("^ ", "%", " a ")
        '%a '
        >>> re.sub("^ ", "%", "  a  ")
        '% a  '
        >>> re.sub(" $", "%", "  a  ")
        '  a %'
        >>> re.sub(" $", "%", "a")
        'a'
        >>> re.sub("^ ", "%", "a")
        'a'
        >>> s = "  s  "
        >>> re.sub("^ ", "%", s)
        '% s  '
        """
        like_search = re.sub("^ ", "%", search)
        like_search = re.sub(" $", "%", like_search)
        logger.debug("search=\"{}\"".format(search))
        logger.debug("like_search=\"{}\"".format(like_search))
        return like_search

    def _make_sql(self, search):
        if self._isascii(search):
            column = "key"
        else:
            column = "value"
        like_search = self._make_like(search)

        _sql = '''{} collate nocase like ?'''.format(column)
        sql = self._sql_template.format(_sql)

        return sql, like_search

    def _make_sql_template(self, table_name, limit):
        _from_where = "* from {} where".format(table_name)
        _limit = "limit {}".format(limit)

        self._sql_template = " ".join((_from_where, "{}", _limit))

    def __iter__(self):
        self._i = 0
        return self

    def __next__(self):
        self._i += 1
        if self._i > self._limit:
            self.named_rows.cur.close()
            raise StopIteration
        nx = self.named_rows.__next__()
        return '|'.join((nx.key, nx.value))

    def _isascii(self, s):
        try:
            s.encode('ascii')
        except UnicodeEncodeError as raiz:
            if raiz.args[0] == 'ascii' and \
               raiz.args[4] == 'ordinal not in range(128)':
                return False
            else:
                raise UnicodeEncodeError(*raiz.args)
        return True

def main(cls, script_name, table_name, log_level=logger.INFO):
    if "--debug" in sys.argv:
        log_level = logger.DEBUG
        start_logger(script_name, log_dir=os.path.curdir, log_level=log_level)

    kv_merged, kv_defaults, kv_argment = \
        merge_kv_by_defaults_and_argument(cls.DEFAULTS)

    kv = set_kv_for_regular(kv_defaults, kv_argment, kv_merged["conf"], table_name)

    if not os.path.isfile(kv["dbpath"]):
        raise OSError("cannot access \"{}\": No such file.".format(kv["dbpath"]))

    logger.debug("cls={}(dbpath={}, table_name={}, mode={}, limit={})".format(cls, kv["dbpath"], table_name, kv["mode"], kv["limit"]))
    iasap_obj = cls(kv["dbpath"], table_name, kv["mode"], kv["limit"])
    logger.debug("iasap_obj = {}".format(iasap_obj))
    if kv["mode"] == "one-shot":
        if not len(kv["args"]):
            msg = ("mode として one-shot を指定した場合は、"
                   "one-shot の後に空白を一つ入れ、空白の後に検索語を一つ"
                   "指定しなければなりません。")
            raise ValueError(msg)
        iasap_obj.start(kv["args"][0])
    else:
        iasap_obj.start()



# Copyright 2011-2015 梅濁酒(umedoblock)

import argparse
import re
import tkinter
import tkinter.scrolledtext
import sys, os

_here = os.path.abspath(__file__)
_parent_dir = os.path.join(os.path.dirname(_here), "..")
# print("_parent_dir =", _parent_dir)
sys.path.append(_parent_dir)

from iasap.lib import logger, start_logger

class IasapTkinter(object):
    MAX_LENGTH = 80
    MAX_HEIGHT = 30

    def __init__(self):
        self.root = tkinter.Tk()
        self.used_query = ""
        self.root.option_add("*font", ("Ricty", 14))
        self.root.title("IasapTkinter")
        self.ctrl_key = False
        self.need_to_search = False

        line = " " * self.MAX_LENGTH
        self.buffer_oneline = tkinter.StringVar()
        self.buffer_oneline.set(line)

        # labelの設定
        oneline = tkinter.Text(self.root,
                            height=1,
                            )
        self.oneline = oneline
        oneline.pack(anchor=tkinter.NW,
                  padx=4,
                  )
      # oneline.bind("<Control-KeyPress>", self.ctrl_key_press)
      # oneline.bind("<Control-KeyRelease>", self.ctrl_key_release)
        oneline.bind("<Any-KeyRelease>", self.release_key)
      # oneline.bind("<1>", self.focus_oneline)
        oneline.focus_set()

        # help(tkinter.Widget)
        # help(tkinter.Text)
        # help(tkinter.scrolledtext.ScrolledText)
        body = tkinter.Text( self.root,
                            borderwidth=1,
                          # state=tkinter.NORMAL,
                          # state=tkinter.DISABLED,
                            bg="gray",
                            fg="black",
                            selectbackground="white",
                            wrap=tkinter.WORD,
                            height=self.MAX_HEIGHT,
                            width=self.MAX_LENGTH,
                            relief=tkinter.RIDGE,
                            font=("Helvetica", 12),
                            )
        body.pack(
                 fill=tkinter.BOTH,
                 expand=1,
               # padx=5, pady=5,
               # ipadx=3, ipady=3,
               # side=tkinter.TOP,
                 )
        self.body = body

    def focus_oneline(self, event):
        self.oneline.focus_set()

    def get_key(self, key):
        key = key.encode()
        if key == b"^C":
            self.root.quit()
        if key == b"^H":
            len_ss = len(self.ss)
            self.ss = self.ss[:len_ss-1]
        else:
            self.ss += key
        cursor_x = len(self.ss)
        if cursor_x > self.MAX_LENGTH - 1:
            cursor_x = self.MAX_LENGTH - 1
            self.ss = self.ss[:cursor_x]
        oneline = self.ss
        padding = b" " * (self.MAX_LENGTH - cursor_x)
        oneline = self.ss + padding
        body = key
        self._set_msg(oneline, body, cursor_x)

    def _set_msg(self, oneline, body, cursor_x):
        logger.debug(".index(INSERT) =", self.txt.index(tkinter.INSERT))
        logger.debug(".index(END) =", self.txt.index(tkinter.END))
        sp = self.txt.index(tkinter.INSERT).split(".")
        index_oneline = "{}.0".format(4)
        self.txt.insert(index_oneline, body + b"\n")

    def to_int(self, ss, sep="."):
        return [int(s) for s in ss.split(sep)]

    def get_oneline(self, event):
        logger.debug("in get_oneline()")
        logger.debug("event.(type={}, keycode={}, keysysm={}, keysym_num={}, "
                     "char=\'{}\')".format(event.type, event.keycode,
                                           event.keysym, event.keysym_num,
                                           event.char))
        # index() が値を返す時、"y.x" の形式で、oneline の左端に cursor
        # がある時、返ってくる文字列は、"1.0"。
        # なので、y=1, x=0 が開始っぽい。変なの。こういう癖、好きじゃない。
        cursor_yx = self.oneline.index(tkinter.INSERT)
        logger.debug("cursor_yx={}".format(cursor_yx))
        line, column = self.to_int(cursor_yx)

        current_oneline = self.oneline.get("1.0", tkinter.END)
        current_oneline = re.sub("\n$", "", current_oneline)

        query = current_oneline
        if line > 1:
            # 現在の oneline が複数行だったら。
            # 現在の oneline から改行を削除して、
            # 改行がない値を oneline に設定し直す。
            query = current_oneline.replace("\n", "")

            self.oneline.delete("1.0", tkinter.END)
            self.oneline.insert(tkinter.END, query)
        if self.used_query != query:
            logger.debug('used_query={}, query={}'.format(self.used_query, query))
            self.used_query = query
            self.need_to_search = True
        else:
            self.need_to_search = False
        logger.debug("need_to_search={}.".format(self.need_to_search))

    def update_body(self, body):
        self.body.delete("1.0", tkinter.END)
        self.body.insert(tkinter.END, body)

    def get_body(self, query):
        body = "\n".join([query]  * self.MAX_HEIGHT)
        return body

    def release_key(self, event):
        self.get_oneline(event)
        if self.need_to_search:
            body = self.get_body(self.used_query)
            self.update_body(body)

    def char_key(self, event):
        logger.debug("char_key() keysym =", event.keysym)
        logger.debug("char_key()   char =", event.char)
        d = {
            "BackSpace": "^H",
            "Delete": "^D",
            "Right": "^F",
            "Left": "^B",
        }
        keysym = event.keysym
        if keysym in d:
            char = d[keysym]
        else:
            char = event.char
        if not re.search("[-a-zA-Z0-9 &|]", char):
            return
        self.get_key(char)

    def ctrl_key_press(self, event):
        self.ctrl_key = True

    def ctrl_key_release(self, event):
        self.ctrl_key = False

    def ctrl_key(self, event):
        if not event.char:
            return
        key = "^" + event.keysym.upper()
        self.get_key(key)

    def start(self):
        try:
            self.root.mainloop()
        except KeyboardInterrupt:
            pass

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="tkinter")
    parser.add_argument("--debug", dest="debug",
                       action="store_true", default=False,
                       help="use debug ? (default: False)")
    args = parser.parse_args()

    log_dir = os.path.curdir
    if args.debug:
        # log_dir が空文字であれば、start_logger() 内部で、
        # log の出力先を sys.stderr に変更する。
        log_dir = ""

    start_logger(__file__, log_dir, logger.DEBUG)
    iat = IasapTkinter()
    iat.start()



import argparse, os, re, shutil

PATTERN = "XXX"

def sed(path, pattern, repl):
    print("sed(path={}, pattern={}, repl={})".format(path, pattern, repl))
    f = open(path, "r")
    tmp_path = path + ".tmp"
    t = open(tmp_path, "w")
    for old_line in f:
        new_line = re.sub(pattern, repl, old_line)
        t.write(new_line)
    t.close()
    f.close()
    os.remove(path)
    os.rename(tmp_path, path)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="init eijiro root directory.")

    parser.add_argument("--xxx", metavar="n", dest="ver",
                       required=True,
                       help=("お使いの英辞郎辞書データの"
                             " Version を指定します。"
                             "「英辞郎辞書データVer98.」をお使いなら、"
                             "--xxx=98 とします。"))
    args = parser.parse_args()

    ver = args.ver
    eijiroXXX = "eijiro" + ver
    eijiroXXX_root = os.path.join("..", eijiroXXX)
    print("eijiroXXX_root =", eijiroXXX_root)

    try:
        shutil.rmtree(eijiroXXX_root)
    except FileNotFoundError as e:
      # print("e =", e)
      # print("e.args =", e.args)
        if e.args == (2, 'No such file or directory'):
            pass
        else:
            raise FileNotFoundError(*e.args)
    os.mkdir(eijiroXXX_root)

    for old_basename in os.listdir("."):
        if old_basename == "init_eijiroXXX.py":
            continue
        if old_basename.endswith("swp"):
            # 糞 Vim !
            continue
        new_basename = re.sub(PATTERN, ver, old_basename)

        old_path = os.path.join(".", old_basename)
        new_path = os.path.join(eijiroXXX_root, new_basename)

        cmd = "shutil.copy({}, {})".format(old_path, new_path)
        shutil.copy(old_path, new_path)
        print(cmd)
        sed(new_path, PATTERN, ver)
        sed(new_path, "OS_SYS_EXECUTE", os.sys.executable)



#!OS_SYS_EXECUTE

# Copyright 2011-2015 梅濁酒(umedoblock)

import os

import lib

lib.sys_path_append_parent_dir(__file__, 2)

__file__ = os.path.abspath(__file__)
dirname = os.path.dirname(__file__)

import iasap

class EijiroXXX(iasap.Iasap):
    DEFAULTS = {
        "conf": os.path.join(dirname, "eijiroXXX.conf"),
        "dbpath": os.path.join(dirname, 'eijiroXXX.sqlite3'),
        "mode": "tkinter",
        "limit": 30,
        "debug": False,
    }

if __name__ == '__main__':
    iasap.main(EijiroXXX, os.path.basename(__file__), "eijiroXXX")



# Copyright 2011-2015 梅濁酒(umedoblock)

import curses
import re
import locale
import os, sys

import lib
lib.sys_path_append_parent_dir(__file__, 1)

from iasap.lib import logger, start_logger

# locale.setlocale(locale.LC_ALL, "")

__all__ = ["IasapCurses"]

class OneLineState(object):
    def __init__(self, mode):
        self.oneline = ""
        self.cursor_x = 0
        self.buffer = ""
        if not mode in ("vim", "emacs"):
            raise ValueError("unknown mode=\"{}\", mode must be \"vim\" or \"emacs\"")
        self.mode = mode

    def update(self, keyname):
        if self.mode == "vim":
            pass
        elif self.mode == "emacs":
            self._mode_emacs(keyname)

        if self.cursor_x < 0:
            self.oneline = ""
            self.cursor_x = 0

    def check_current(self):
        return self.oneline, self.cursor_x

    def _mode_emacs(self, keyname):
        oneline = self.oneline
        cursor_x = self.cursor_x
        buffer = self.buffer

        logger.debug('oneline="{}", cursor_x={}, buffer="{}" enter in _mode_emacs(keyname="{}")'.format(oneline, cursor_x, buffer, keyname))
        if keyname == "^A":
            # ctr-a     "^A"
            cursor_x = 0
        elif keyname == "^K":
            buffer = oneline[cursor_x:]
            oneline = oneline[:cursor_x]
        elif keyname == "^F":
            cursor_x += 1
            if cursor_x > len(oneline):
                cursor_x = len(oneline)
        elif keyname == "^Y":
            oneline = "".join((oneline[:cursor_x], buffer, oneline[cursor_x:]))
        elif keyname == "^T":
            if len(oneline) >= 2 and cursor_x <= len(oneline) - 2:
                c0 = oneline[cursor_x+0]
                c1 = oneline[cursor_x+1]
                left = oneline[:cursor_x]
                right = oneline[cursor_x+2:]
                oneline = left + c1 + c0 + right
        elif keyname == "^E":
            cursor_x = len(oneline)
        elif keyname == "^D":
            oneline = oneline[:cursor_x] + oneline[cursor_x+1:]
        elif keyname == "^B":
            if cursor_x > 0:
                cursor_x -= 1
        elif keyname == "^W":
            if not cursor_x:
                pass
            elif oneline[cursor_x-1] == " ":
                while cursor_x > 0 and oneline[cursor_x-1] == " ":
                    oneline = oneline[:cursor_x-1] + oneline[cursor_x:]
                    cursor_x -= 1
            else:
                while cursor_x > 0 and oneline[cursor_x-1] != " ":
                    oneline = oneline[:cursor_x-1] + oneline[cursor_x:]
                    cursor_x -= 1
        elif keyname in ("^H", "^?"):
            # ctr-h     "^H"
            # backspace "^?"
            if not cursor_x:
                pass
            else:
                oneline = oneline[:cursor_x-1] + oneline[cursor_x:]
                cursor_x -= 1
        elif len(keyname) >= 2:
            pass
        elif re.search("[-a-zA-Z0-9 &|]", keyname):
            # ord("a") => 0x61
            # chr(0x61) => "a"
            oneline = oneline[:cursor_x] + keyname + oneline[cursor_x:]
            cursor_x += 1

        self.oneline = oneline
        self.cursor_x = cursor_x
        self.buffer = buffer

        logger.debug('oneline="{}", cursor_x={}, buffer="{}" leave out _mode_emacs(keyname="{}")'.format(oneline, cursor_x, buffer, keyname))

class IasapCurses(object):
    MAX_LENGTH = 80
    MAX_HEIGHT = 30

    def __init__(self, height=MAX_HEIGHT):
        self.stdscr = curses.initscr()
        curses.noecho()
        self.stdscr.nodelay(False)
        self._height = height
        self._ols = OneLineState("emacs")

    def getch(self, y, x):
        ch = self.stdscr.getch(y, x)
        if ch == 3:
            # enable Ctr-c in windows.
            logger.debug("stdscr.getch() raise KeyboardInterrupt.")
            raise KeyboardInterrupt
        return ch

    def _init_screen(self):
        self.stdscr.erase()

    def _draw_oneline(self, oneline):
        self.stdscr.addstr(0, 0, oneline)

    def _get_new_ch(self, cursor_x):
        try:
            new_ch = self.getch(0, cursor_x)
        except KeyboardInterrupt:
            new_ch = "__break__"

        logger.debug("new_ch = {} in _get_new_ch()".format(new_ch))
        return new_ch

    def _update_oneline(self, oneline, cursor_x, keyname):
        old_oneline, old_cursor_x = self._ols.check_current()
        self._ols.update(keyname)

        new_query = False
        if self._ols.oneline != old_oneline:
            new_query = True

        return self._ols.oneline, self._ols.cursor_x, new_query

    def get_body(self, query):
        body = "\n".join([query] * self._height)
        return body

    def _draw_line(self, line, ss, y, width, effect):
      self.stdscr.addstr(y, 0, line, effect)
      # try:
      #     self.stdscr.addstr(y, 0, line, effect)
      #     return True
      # except curses.error as raiz:
      #     self.stdscr.addch(0, len(ss) - 1, " ")
      #     # ss = ss[:-1]
      #     self.stdscr.addstr(0, 0, ss)
      #     args = raiz.args
      #     message = "raised error reason: {}".format(args[0])
      #     return False
      #     # self.stdscr.addstr(1, 0, message)

    def _effect_line(self, line, ss, y, width, effect):
        # count ascii charactor to ascii_counter
        for ascii_counter, c in enumerate(line):
            if len(c.encode()) > 1:
                break
        ss_lower = ss.lower()
        len_ss = len(ss)

        x = 0
        while True:
            try:
                rel_index = line.lower().index(ss_lower)
            except ValueError as raiz:
                args = raiz.args
                if args[0] == "substring not found":
                    break
                else:
                    logger.debug("args = {}, in _effect_line()".format(args))
                    raise ValueError(*args)
            else:
                x += rel_index
                if x < width and x < ascii_counter:
                    rev = line[rel_index:rel_index+len_ss]
                    self.stdscr.addstr(y, x, rev, effect)
                    x += len_ss
                    line = line[rel_index+len_ss:]
                else:
                    break

    def start(self):
        # for organize with OneLineTkinter class.
        self.run()

    def run(self):
        new_ch_no = 0
        cursor_x = 0
        oneline = ""
        while True:
            logger.debug("new_ch_no = {} ==================".format(new_ch_no))
            ymax, xmax = self.stdscr.getmaxyx()
            new_ch = self._get_new_ch(cursor_x)
            new_ch_no += 1
            if new_ch == "__break__":
                self.stdscr.erase()
                break
            keyname = curses.keyname(new_ch)
            logger.debug("keyname = \"{}\", len(keyname) = {}, type(keyname) = {}, in run()".format(keyname, len(keyname), type(keyname)))
            keyname = keyname.decode()
            logger.debug("keyname = \"{}\", len(keyname) = {}, type(keyname) = {}, in run()".format(keyname, len(keyname), type(keyname)))
            oneline, cursor_x, new_query = \
                self._update_oneline(oneline, cursor_x, keyname)

            if not len(oneline) < xmax:
                oneline = oneline[:xmax - 1]
                cursor_x -= 1

            if not oneline and not cursor_x:
                self.stdscr.erase()
            if re.search("^ *$", oneline) or not new_query:
                continue

            self._init_screen()
            self._draw_oneline(oneline)

            height = ymax - 1
            width = xmax
            curses_lines = self.get_body(oneline)
            lines = curses_lines.split("\n")
            offset_y = 1
            for i, line in enumerate(lines):
              # logger.debug("i={}, line={}".format(i, line)) OK
                try:
                    self._draw_line(line, oneline, i + offset_y, \
                                       width, curses.A_NORMAL)
                except curses.error as raiz:
                    break

        self.stdscr.erase()
        curses.nocbreak()
        self.stdscr.keypad(0)
        curses.echo()
        curses.endwin()
        logger.debug("oneline_curses finished.")

if __name__ == "__main__":
    start_logger(__file__, os.path.curdir, logger.DEBUG)

    iac = IasapCurses(20)
    iac.run()



# Copyright 2011-2015 梅濁酒(umedoblock)

import sqlite3
import argparse
import configparser
import re
import os

import lib
lib.sys_path_append_parent_dir(__file__, 2)
from iasap.sql import GeneralSQLConnection
from iasap.lib import start_logger, logger

from eijiroXXX import EijiroXXX

start_logger(os.path.basename(__file__), os.path.curdir, logger.INFO)

parser = argparse.ArgumentParser(description='make eijiroXXX database.')

parser.add_argument('--txtpath', metavar='f', dest='txtpath',
                    required=True,
                    help='eijiroXXX.txt path')
parser.add_argument('--table-name', metavar='t', dest='table_name',
                    nargs='?',
                    default='eijiroXXX',
                    help='default: eijiroXXX')
parser.add_argument('--dbpath', metavar='f', dest='dbpath',
                    nargs='?',
                    default=EijiroXXX.DEFAULTS["dbpath"],
                    help='default: {}'.format(EijiroXXX.DEFAULTS["dbpath"]))
parser.add_argument('--conf', metavar='f', dest='conf',
                    nargs='?',
                    default=EijiroXXX.DEFAULTS["conf"],
                    help='default: {}'.format(EijiroXXX.DEFAULTS["conf"]))
parser.add_argument('--debug', dest='debug',
                   action='store_true', default=False,
                   help='use debug ? (default: False)')

args = parser.parse_args()
txtpath = args.txtpath
print('args.txtpath =', txtpath)
if not txtpath:
    raise ValueError('txtpath muse be available txtpath')
dbpath = args.dbpath
print('args.dbpath =', dbpath)
if not dbpath:
    raise ValueError('dbpath muse be available path')
conf = args.conf
print('args.conf =', conf)
if not conf:
    raise ValueError('conf muse be available path')

txtpath = os.path.expanduser(txtpath)
dbpath = os.path.expanduser(dbpath)
confpath = os.path.expanduser(conf)

if os.path.isfile(dbpath):
    print("dbpath として指定した path には既に、file が存在します。")
    print(dbpath)
    s = input("を削除しますか？ 削除するなら、yes を入力してください。->: ")

    if s.lower() == "yes":
        os.remove(dbpath)
        print("dbpath={}\nを削除しました。".format(dbpath))
    else:
        raise ValueError("起動 option --dbpath=xxx に適切な path を入力して下さい。")

config = configparser.ConfigParser()
config.read(confpath)
conn_sqlite3 = sqlite3.connect(dbpath)
conn = GeneralSQLConnection(conn_sqlite3)
try:
    conn.create_table(config['schema'])
except sqlite3.OperationalError as raiz:
    if raiz.args[0] != 'table {} already exists'.format(args.table_name):
        raise sqlite3.OperationalError(*raiz.args)

print()
print("dbpath={}\nに、sqlite3 形式の db を作成しています。".format(dbpath))
with open(txtpath, encoding='utf8') as f:
    for l in f:
        l = l.strip()
      # print(l)
        key, value = re.findall('(.*) /// (.*)', l)[0]
      # prefix, v0 = parse_key(key)
      # v1, suffix = parse_value(value)
      # print(prefix, v0, v1, suffix, sep='|')
      # print(key, value, sep='|')
        column = \
            {'id': None, 'key': key, 'value': value}
        conn.insert(args.table_name, column)
conn.close()



# iasap の setup.py

from distutils.core import setup

setup(
    name = "iasap",
    packages = ["iasap"],
    version = "0.0.2",
    description = "interactive as sonn as possible",
    author = "梅濁酒(umedoblock)",
    author_email = "umedoblock@gmail.com",
    url='http://pypi.python.org/pypi/iasap/',
    download_url = "",
    license='MIT License',
    platforms=['unix', 'linux', 'osx', 'cygwin', 'win32'],
    keywords=['tkinter', 'curses', 'sqlite3', 'key-value store',
              'incremental search'],
    # see
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        'Programming Language :: Python :: 3',
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Environment :: Console :: Curses',
        'Environment :: Win32 (MS Windows)',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Japanese',
        'Operating System :: OS Independent',
        'Topic :: Education',
    ],
    long_description = """\
| interactive as soon as possible (a.k.a iasap)
| --------------------------------------------
| iasap は、key-value store 形式のテキスト検索を目的として開発しました。
| iasap は、incremental 検索機能を実現しています。
| 
| iasap は、key-value store の key, value の保存先として、sqlite3 を使用していま
| す。
| iasap は、tkinter, curses, command prompt のいずれかで、対話的に検索できます。
| iasap は、検索単語に空白を入力せずに検索すると、一致検索を実行します。
| iasap は、検索単語の前に空白を入力すると前方一致検索、 検索単語の後に空白を入力
| すると後方一致検索、を実行します。
| 
| 開発者の私は、手元の PC で英辞郎を使いたくて、
| iasap を開発しました。
| 英辞郎を PC で辞書として使う方法について詳しくは、
| 同梱の README.txt をご覧下さい。
| 
| このバージョンの iasap は、Python3.x で使用してください。
"""
)



import sys, os
import datetime, argparse

import logging as _logger
import configparser

global logger
logger = _logger

CHARACTORS_PRINTED_PER_LINE = 79
CPPL = CHARACTORS_PRINTED_PER_LINE

def sys_path_append_parent_dir(file_path, n_up):
    if os.path.islink(file_path):
        file_path = os.path.realpath(file_path)
    here = os.path.abspath(file_path)
    ups = ""
    if n_up:
        ups = os.path.join(*([".."] * n_up))
    parent_dir = os.path.join(os.path.dirname(here), ups)
  # print("parent_dir =", parent_dir)
    sys.path.append(parent_dir)

def start_logger(script_name, log_dir=os.path.curdir, log_level=logger.INFO):
    # 通常使用時は、log_dir=os.path.curdir を想定している。
    # log_dir が空文字であれば、log の出力先を sys.stderr に変更する。

    basename = os.path.basename(script_name)
    if log_dir:
        log_base = ".".join((basename, "log"))
        log_path = os.path.join(log_dir, log_base)

        # http://docs.python.jp/3.4/library/logging.html#logging.basicConfig
        # stream: 指定されたストリームを StreamHandler の初期化に使います。
        # この引数は 'filename' と同時には使えないことに注意してください。
        # 両方が指定されたときには ValueError が送出されます。
        # logger.basicConfig(filename='/dev/null', level=logger.INFO)
        # logger.basicConfig(stream=sys.stderr, level=logger.DEBUG)
        logger.basicConfig(filename=log_path, level=log_level)
    else:
        logger.basicConfig(stream=sys.stderr, level=log_level)

    log_prefix = "INFO:root:"
    horizontal = "=" * (CPPL - len(log_prefix))
    _now = datetime.datetime.now()
    body = "{} start! at {} ".format(basename, _now)
    value = "=" * (CPPL - (len(log_prefix) + len(body)))

    logger.info(horizontal)
    logger.info(body + value)
    logger.info(horizontal)

def _merge_kv(first, second, third={}):
    kv = {}
    for k, v in first.items():
        kv[k] = v
    for k, v in second.items():
        kv[k] = v
    for k, v in third.items():
        kv[k] = v
    logger.debug("kv in _merge_kv() =")
    logger.debug(kv)
    logger.debug("\n")
    return kv

def _get_kv_by_conf(path, section):
    logger.debug("path =")
    logger.debug(path)
    with open(path) as f:
        config = configparser.ConfigParser()
        config.read_file(f, section)
    kv = {}
    for key, value in config[section].items():
        if value:
            kv[key] = value
    if kv["limit"]:
        kv["limit"] = int(kv["limit"])
    logger.debug("kv in _get_kv_by_conf() =")
    logger.debug(kv)
    logger.debug("\n")
    return kv

def _set_kv_by_defaults(defaults=None):
    logger.debug("kv in _set_kv_by_defaults() =")
    logger.debug(defaults)
    logger.debug("\n")
    return defaults

def merge_kv_by_defaults_and_argument(defaults=None):
    kv_defaults = _set_kv_by_defaults(defaults)
    kv_argment = _get_kv_by_argument(defaults)
    kv_tmp = _merge_kv(kv_defaults, kv_argment)
    return kv_tmp, kv_defaults, kv_argment

def set_kv_for_regular(kv_defaults, kv_argment, conf_path, section):
    kv_conf = _get_kv_by_conf(conf_path, section)
    kv = _merge_kv(kv_defaults, kv_conf, kv_argment)
    return kv

def _get_kv_by_argument(defaults=None):
    parser = argparse.ArgumentParser(description='argment.')

    parser.add_argument('--conf', metavar='f', dest='conf',
                       default=defaults["conf"],
                       help='default conf path is {}.'.format(defaults["conf"]))
    parser.add_argument('--dbpath', metavar='f', dest='dbpath',
                       default=defaults["dbpath"],
                       help='default sqlite dbpath is {}'.format(defaults["dbpath"]))
    parser.add_argument('--mode', metavar='s', dest='mode',
                       default=defaults["mode"],
                       help='default iasap library mode is tkinter, curses or one-shot.')
    parser.add_argument("--limit", metavar="N", dest="limit",
                         type=int, default=defaults["limit"],
                         help="max number of rows as select result, default is {}".format(defaults["limit"]))
    parser.add_argument("--debug", help="debug mode switch, default is False.", action="store_true")
    parser.add_argument('args', nargs=argparse.REMAINDER)

    args = parser.parse_args()
    kv = {}
    logger.debug("sys.argv =")
    logger.debug(sys.argv)
    logger.debug("args =")
    logger.debug(args)
    logger.debug("vars(args) =")
    logger.debug(vars(args))
#   logger.debug("args.items() =")
#   logger.debug(args.items())
    shell_command = "`".join(sys.argv)
    for k, v in vars(args).items():
        logger.debug("k={}, v={}".format(k, v))
        if "--{}".format(k) in shell_command:
            # user $B;XDj(B
            kv[k] = v
    kv["args"] = args.args
    logger.debug("kv in _get_kv_by_argument() =")
    logger.debug(kv)
    logger.debug("\n")
    return kv




# Copyright 2011-2015 梅濁酒(umedoblock)

import re
import tkinter

class IasapTkinter(object):
    def __init__(self):
        self.root = tkinter.Tk()
        self.ss = b''
        self.root.option_add('*font', ('Ricty', 14))
        self.root.title('IasapTkinter')

        self.buffer = tkinter.StringVar()
        self.buffer.set(' ' * 80)

        # labelの設定
        lbl = tkinter.Label(self.root,
                            text='*** push any key ***RAISED',
                            background='#00ff00',
                            underline=1,
                            relief=tkinter.RAISED,
                            )
        lbl.pack(anchor=tkinter.W,
                 padx=5, pady=5,
                 ipadx=3, ipady=3,
                 side=tkinter.TOP,
                 )
        lbl = tkinter.Label(self.root,
                            text='*** push any key ****RIDGE',
                            background='gray',
                            underline=1,
                            borderwidth=3,
                            relief=tkinter.RIDGE,
                            )
        lbl.pack(anchor=tkinter.W,
                 padx=5, pady=5,
                 ipadx=3, ipady=3,
                 side=tkinter.TOP,
                 )
        lbl = tkinter.Label(self.root,
                            text='*** push any key ***SUNKEN',
                            background='#0000ff',
                            underline=1,
                            relief=tkinter.RAISED,
                            )
        lbl.pack(anchor=tkinter.W,
                 padx=5, pady=5,
                 ipadx=3, ipady=3,
                 side=tkinter.TOP,
                 )
        lbl = tkinter.Label(self.root,
                            text='*** push any key ***GROOVE',
                            background='red',
                            underline=1,
                            relief=tkinter.GROOVE,
                            )
        lbl.pack(anchor=tkinter.W,
                 padx=5, pady=5,
                 ipadx=3, ipady=3,
                 side=tkinter.TOP,
                 )

        a = tkinter.Label(self.root,
                          textvariable=self.buffer,
                          underline=0,
                          )
        self.a = a
        a.pack(anchor=tkinter.NW)
        a.bind('<Control-KeyPress>', self.ctrl_key)
        a.bind('<Any-KeyPress>', self.char_key)
        a.focus_set()

        line = ' ' * 80 + '\n'
        lbl = tkinter.Label(self.root,
                            text=line * 20,
                            borderwidth=1,
                            relief=tkinter.RIDGE,
                            )
        lbl.pack(anchor=tkinter.W,
                 padx=5, pady=5,
                 ipadx=3, ipady=3,
                 side=tkinter.TOP,
                 )

    def get_key(self, key):
        key = key.encode()
        if key == b'^C':
            self.root.quit()
        self.ss += key
        len_ss = len(self.ss)
      # msg = b'get_key() push key is ' + self.ss
      # msg = b' ' * 80 + b'\n' + self.ss
        padding = b' ' * (80 - len_ss)
        msg = self.ss + padding
        # print(help(self.buffer))
        self.a.config(underline=len_ss)
        self.buffer.set(msg)

    def char_key(self, event):
        char = event.char
        if not re.search('[-a-zA-Z0-9 &|]', char):
            return
        self.get_key(char)

    def ctrl_key(self, event):
        key = '^' + event.keysym.upper()
        self.get_key(key)

    def start(self):
        try:
            self.root.mainloop()
        except KeyboardInterrupt:
            pass

if __name__ == '__main__':
    olt = IasapTkinter()
    olt.start()



# Copyright 2011-2015 梅濁酒(umedoblock)

import unittest

from sql import *
print(dir())

class TestSQL(unittest.TestCase):
    def test_sql(self):
        pass

if __name__ == '__main__':
    unittest.main()
